import * as React from 'react';
import { StyleSheet } from 'react-native';
import { cacheStyles, getTheme, useTheme } from '../services/ThemeContext';
/**
 * Creates a styled component using a `styler` parameter. The `styler` can be the
 * styles itself, a function with the current theme and component props as
 * curried arguments (e.g. `theme => props => styles` or `theme => styles`).
 * The styles are cached when no `props` parameter is required by the `styler`.
 *
 * **Example Usage**
 *
 * ```
 * const Colorful = styled(Text)({ fontWeight: 'Red' })
 * const Themed = styled(Text)(theme => ({ color: theme.color }))
 * const Dynamic = styled(Text)(theme => props => ({ color: props.color ?? theme.color }))
 * const DynamicSansTheme = styled(Text)(_theme => props => ({ color: props.color }))
 * ```
 */
export function styled(Component) {
    function makeStyledComponent(styler) {
        function addName(StyledComponent) {
            StyledComponent.displayName =
                Component.displayName != null
                    ? `StyledComponent(${Component.displayName})`
                    : `StyledComponent`;
            return StyledComponent;
        }
        if (typeof styler === 'function') {
            const rv = styler(getTheme());
            if (typeof rv === 'function') {
                const stylerNarrowed = styler;
                return addName(function StyledComponent(props) {
                    const theme = useTheme();
                    const style = stylerNarrowed(theme)(props);
                    const allProps = Object.assign(Object.assign({}, props), { style: style });
                    return React.createElement(Component, Object.assign({}, allProps));
                });
            }
            else {
                const stylerNarrowed = styler;
                const getStyles = cacheStyles((theme) => ({
                    style: stylerNarrowed(theme)
                }));
                return addName(function StyledComponent(props) {
                    const theme = useTheme();
                    const stylesheet = getStyles(theme);
                    const allProps = Object.assign(Object.assign({}, props), { style: stylesheet.style });
                    return React.createElement(Component, Object.assign({}, allProps));
                });
            }
        }
        else {
            const stylesheet = StyleSheet.create({ style: styler });
            return addName(function StyledComponent(props) {
                const allProps = Object.assign(Object.assign({}, props), { style: stylesheet.style });
                return React.createElement(Component, Object.assign({}, allProps));
            });
        }
    }
    return makeStyledComponent;
}
export function styledWithRef(Component) {
    function makeStyledComponent(styler) {
        function addName(StyledComponentWithRef) {
            StyledComponentWithRef.displayName =
                Component.displayName != null
                    ? `StyledComponentWithRef(${Component.displayName})`
                    : `StyledComponentWithRef`;
            return StyledComponentWithRef;
        }
        if (typeof styler === 'function') {
            const rv = styler(getTheme());
            if (typeof rv === 'function') {
                const stylerNarrowed = styler;
                return addName(React.forwardRef(function StyledComponent(props, ref) {
                    const theme = useTheme();
                    const style = stylerNarrowed(theme)(props);
                    const allProps = Object.assign(Object.assign({}, props), { style });
                    return React.createElement(Component, Object.assign({}, allProps, { ref: ref }));
                }));
            }
            else {
                const stylerNarrowed = styler;
                const getStyles = cacheStyles((theme) => ({
                    style: stylerNarrowed(theme)
                }));
                return addName(React.forwardRef(function StyledComponent(props, ref) {
                    const theme = useTheme();
                    const stylesheet = getStyles(theme);
                    const allProps = Object.assign(Object.assign({}, props), { style: stylesheet.style });
                    return React.createElement(Component, Object.assign({}, allProps, { ref: ref }));
                }));
            }
        }
        else {
            const stylesheet = StyleSheet.create({ style: styler });
            return addName(React.forwardRef(function StyledComponent(props, ref) {
                const allProps = Object.assign(Object.assign({}, props), { style: stylesheet.style });
                return React.createElement(Component, Object.assign({}, allProps, { ref: ref }));
            }));
        }
    }
    return makeStyledComponent;
}
