/**
 * IMPORTANT: Changes in this file MUST be synced with edge-react-gui!
 */
import * as React from 'react';
import { BackHandler, Dimensions, View } from 'react-native';
import { Gesture, GestureDetector, ScrollView } from 'react-native-gesture-handler';
import { cacheStyles } from 'react-native-patina';
import Animated, { runOnJS, useAnimatedStyle, useSharedValue, withTiming } from 'react-native-reanimated';
import AntDesignIcon from 'react-native-vector-icons/AntDesign';
import { useHandler } from '../../hooks/useHandler';
import { EdgeTouchableOpacity } from '../common/EdgeTouchableOpacity';
import { EdgeTouchableWithoutFeedback } from '../common/EdgeTouchableWithoutFeedback';
import { useTheme } from '../services/ThemeContext';
import { EdgeText } from '../themed/EdgeText';
import { BlurBackground } from './BlurBackground';
const BACKGROUND_ALPHA = 0.7;
const safeAreaGap = 64; // Overkill to avoid bottom of screen
const duration = 300;
/**
 * A modal that slides a modal up from the bottom of the screen
 * and dims the rest of the app.
 */
export function ModalUi4(props) {
    const { bridge, title, children, noSwiping = false, scroll = false, warning = false, onCancel } = props;
    const theme = useTheme();
    const styles = getStyles(theme);
    const halfRem = theme.rem(0.5);
    const closeThreshold = theme.rem(6);
    const dragSlop = theme.rem(1);
    //
    // Shared state
    //
    const opacity = useSharedValue(0);
    const offset = useSharedValue(Dimensions.get('window').height);
    const handleCancel = useHandler(() => {
        if (onCancel != null)
            onCancel();
    });
    //
    // Effects
    //
    React.useEffect(() => bridge.on('clear', handleCancel), [
        bridge,
        handleCancel
    ]);
    React.useEffect(() => {
        // Animate in:
        opacity.value = withTiming(BACKGROUND_ALPHA, { duration });
        offset.value = withTiming(0, { duration });
        // Animate out:
        bridge.on('result', () => {
            opacity.value = withTiming(0, { duration });
            offset.value = withTiming(Dimensions.get('window').height, { duration }, () => runOnJS(bridge.remove)());
        });
    }, [bridge, opacity, offset]);
    React.useEffect(() => {
        const backHandler = BackHandler.addEventListener('hardwareBackPress', () => {
            handleCancel();
            return true;
        });
        return () => backHandler.remove();
    }, [handleCancel]);
    const gesture = Gesture.Pan()
        .enabled(!noSwiping)
        .onUpdate(e => {
        offset.value = e.translationY;
    })
        .onEnd(() => {
        if (offset.value > closeThreshold) {
            runOnJS(handleCancel)();
        }
        offset.value = withTiming(0, { duration });
    });
    //
    // Dynamic styles
    //
    const underlayStyle = useAnimatedStyle(() => ({
        opacity: opacity.value
    }));
    const modalStyle = useAnimatedStyle(() => ({
        transform: [{ translateY: Math.max(-dragSlop, offset.value) }]
    }));
    const bottomGap = safeAreaGap + dragSlop;
    const isHeaderless = title == null && onCancel == null;
    const isCustomTitle = title != null && typeof title !== 'string';
    const modalLayout = {
        borderColor: warning ? theme.warningText : theme.modalBorderColor,
        borderWidth: warning ? 4 : theme.modalBorderWidth,
        marginBottom: -bottomGap,
        paddingTop: isHeaderless ? halfRem : 0,
        paddingBottom: bottomGap + (scroll ? 0 : halfRem) // Ignore padding on bottom for scrollable content so we don't have a cutoff gap when scrolling content
    };
    return (React.createElement(React.Fragment, null,
        React.createElement(EdgeTouchableWithoutFeedback, { onPress: handleCancel },
            React.createElement(Animated.View, { style: [styles.underlay, underlayStyle] })),
        React.createElement(GestureDetector, { gesture: gesture },
            React.createElement(Animated.View, { style: [styles.modal, modalStyle, modalLayout] },
                React.createElement(BlurBackground, null),
                React.createElement(View, { style: styles.dragBarContainer },
                    React.createElement(View, { style: styles.dragBar })),
                isHeaderless ? null : (React.createElement(View, { style: styles.titleContainer },
                    typeof title === 'string' ? (React.createElement(EdgeText, { style: styles.titleText, numberOfLines: 2 }, title)) : (title !== null && title !== void 0 ? title : undefined),
                    onCancel == null ? null : (React.createElement(EdgeTouchableOpacity, { style: isCustomTitle
                            ? styles.closeIconContainerAbsolute
                            : styles.closeIconContainer, onPress: onCancel },
                        React.createElement(AntDesignIcon, { name: "close", color: theme.deactivatedText, size: theme.rem(1.25) }))))),
                scroll ? (React.createElement(ScrollView, { style: styles.scroll }, children)) : (children)))));
}
const getStyles = cacheStyles((theme) => ({
    underlay: {
        bottom: 0,
        left: 0,
        position: 'absolute',
        right: 0,
        top: 0,
        backgroundColor: theme.modalSceneOverlayColor
    },
    modal: {
        alignSelf: 'flex-end',
        backgroundColor: theme.modalBackground,
        borderTopLeftRadius: theme.rem(1),
        borderTopRightRadius: theme.rem(1),
        flexShrink: 1,
        overflow: 'hidden',
        width: theme.rem(30),
        paddingLeft: theme.rem(0.5),
        paddingRight: theme.rem(0.5)
    },
    scroll: {
        // Only take up as much space as needed to display the contents
        flexGrow: 0,
        flexShrink: 1
    },
    dragBarContainer: {
        alignItems: 'center',
        position: 'absolute',
        left: 0,
        right: 0,
        top: 0
    },
    dragBar: {
        backgroundColor: theme.modalDragbarColor,
        borderRadius: theme.rem(0.125),
        height: theme.rem(0.25),
        marginTop: theme.rem(0.25),
        width: theme.rem(3)
    },
    closeIconContainer: {
        // Used when this component is managing the title
        flexGrow: 1,
        alignSelf: 'flex-start',
        alignItems: 'flex-end',
        justifyContent: 'flex-start',
        paddingTop: theme.rem(0.15),
        marginRight: theme.rem(0.25) // Less margins because the icon itself comes with whitespace
    },
    closeIconContainerAbsolute: {
        // Used when the caller passes a special title that may span the entire
        // width. It's up to the caller to ensure there's no overlap with the close button.
        position: 'absolute',
        top: theme.rem(0.15),
        right: theme.rem(0.25)
    },
    titleContainer: {
        flexDirection: 'row',
        alignItems: 'flex-start',
        marginTop: theme.rem(1.25),
        marginBottom: theme.rem(0.5)
    },
    titleText: {
        fontFamily: theme.fontFaceMedium,
        fontSize: theme.rem(1.2),
        marginHorizontal: theme.rem(0.5),
        flexShrink: 1
    }
}));
