var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { NativeModules, Platform } from 'react-native';
import { getKeychainStatus, readKeychainFile, saveKeychainStatus } from './util/keychainFile';
const nativeMethods = NativeModules.AbcCoreJsUi;
/**
 * Should be called at login to ensure biometric login is properly enabled.
 * @param account The account that has just logged in.
 */
export function refreshTouchId(account) {
    return __awaiter(this, void 0, void 0, function* () {
        const supported = yield supportsTouchId();
        const file = yield readKeychainFile();
        const status = getKeychainStatus(file, account);
        if (supported && status !== false) {
            const loginKey = yield account.getLoginKey();
            const location = typeof status === 'string' ? status : account.rootLoginId + '_loginId';
            yield nativeMethods.setKeychainString(loginKey, location);
            // Update the file:
            yield saveKeychainStatus(file, account, location);
        }
    });
}
export function isTouchEnabled(account) {
    return __awaiter(this, void 0, void 0, function* () {
        const supported = yield supportsTouchId();
        const file = yield readKeychainFile();
        const status = getKeychainStatus(file, account);
        return supported && typeof status === 'string';
    });
}
export function enableTouchId(account) {
    return __awaiter(this, void 0, void 0, function* () {
        const supported = yield supportsTouchId();
        const file = yield readKeychainFile();
        const status = getKeychainStatus(file, account);
        if (!supported) {
            throw new Error('TouchIdNotSupportedError');
        }
        const loginKey = yield account.getLoginKey();
        const location = typeof status === 'string' ? status : account.rootLoginId + '_loginId';
        yield nativeMethods.setKeychainString(loginKey, location);
        // Update the file:
        yield saveKeychainStatus(file, account, location);
    });
}
export function disableTouchId(account) {
    return __awaiter(this, void 0, void 0, function* () {
        const supported = yield supportsTouchId();
        const file = yield readKeychainFile();
        const status = getKeychainStatus(file, account);
        if (supported && typeof status === 'string') {
            yield nativeMethods.clearKeychain(status);
        }
        // Update the file:
        yield saveKeychainStatus(file, account, false);
    });
}
export function getSupportedBiometryType() {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const supported = yield supportsTouchId();
            if (!supported)
                return false;
            const biometryType = yield nativeMethods.getSupportedBiometryType();
            switch (biometryType) {
                // Keep these as-is:
                case 'FaceID':
                case 'TouchID':
                    return biometryType;
                // Android sends this one:
                case 'Fingerprint':
                    return 'TouchID';
                // Translate anything truthy to 'TouchID':
                default:
                    return biometryType ? 'TouchID' : false;
            }
        }
        catch (error) {
            console.log(error);
            return false;
        }
    });
}
/**
 * Looks up the stored biometric secret for a user.
 * Returns undefined if there is no secret, or if the user denies the request.
 */
export function getLoginKey(userInfo, promptString, fallbackString) {
    return __awaiter(this, void 0, void 0, function* () {
        const supported = yield supportsTouchId();
        const file = yield readKeychainFile();
        const status = getKeychainStatus(file, userInfo);
        if (!supported || typeof status !== 'string') {
            return;
        }
        if (Platform.OS === 'ios') {
            const loginKey = yield nativeMethods.getKeychainString(status);
            if (typeof loginKey !== 'string' || loginKey.length <= 10) {
                console.log('No valid loginKey for TouchID');
                return;
            }
            console.log('loginKey valid. Launching TouchID modal...');
            const success = yield nativeMethods.authenticateTouchID(promptString, fallbackString);
            if (success)
                return loginKey;
            console.log('Failed to authenticate TouchID');
        }
        else if (Platform.OS === 'android') {
            try {
                return yield nativeMethods.getKeychainStringWithFingerprint(status, promptString);
            }
            catch (error) {
                console.log(error); // showError?
            }
        }
    });
}
function supportsTouchId() {
    return __awaiter(this, void 0, void 0, function* () {
        if (nativeMethods == null) {
            console.warn('Native edge-login-ui-rn methods are missing');
            return false;
        }
        const out = yield nativeMethods.supportsTouchId();
        return !!out;
    });
}
